using skydiveLogs_api.Domain;
using System.Collections.Generic;

namespace skydiveLogs_api.DomainBusiness.Interfaces
{
    public interface IStatsService
    {
        #region Public Methods

        SimpleSummary GetSimpleSummary();

        IEnumerable<Statistic> GetStatsByAircraft();

        IEnumerable<Statistic> GetStatsByDz();

        IEnumerable<Statistic> GetStatsByGear();

        IEnumerable<Statistic> GetStatsByJumpType();

        IEnumerable<Statistic> GetStatsByYear();

        IEnumerable<Statistic> GetStatsForLastMonthByDz();

        IEnumerable<Statistic> GetStatsForLastMonthByJumpType();

        IEnumerable<Statistic> GetStatsForLastYearByDz();

        IEnumerable<Statistic> GetStatsForLastYearByJumpType();

        #endregion Public Methods
    }
}